@@ -8,7 +8,7 @@
  *	New quota format implementation - Jan Kara <jack@suse.cz> - Sponsored by SuSE CR
  */
 
-#ident "$Id: quotacheck.c,v 1.34 2003/04/08 13:11:06 jkar8572 Exp $"
+#ident "$Id: quotacheck.c,v 1.38 2004/02/08 16:34:17 jkar8572 Exp $"
 
 #include <dirent.h>
 #include <stdio.h>
@@ -28,7 +28,6 @@
 #include <sys/mount.h>
 
 #if defined(EXT2_DIRECT)
-#include <linux/ext2_fs.h>
 #include <ext2fs/ext2fs.h>
 #endif
 
@@ -199,7 +198,7 @@ static void add_to_quota(int type, ino_t i_num, uid_t i_uid, gid_t i_gid, mode_t
 		if (store_dlinks(type, i_num))	/* Did we already count this inode? */
 			return;
 	lptr->dq_dqb.dqb_curinodes++;
-	lptr->dq_dqb.dqb_curspace += i_space;
+	lptr->dq_dqb.dqb_curspace += i_space;;
 }
 
 /*
@@ -402,7 +401,7 @@ static int ext2_direct_scan(char *device)
 
 	while ((long)i_num) {
 		if (inode.i_links_count) {
-			debug(FL_DEBUG, _("Found i_num %ld, %ld\n"), (long)i_num, (long)inode.i_blocks);
+			debug(FL_DEBUG, _("Found i_num %ld, blocks %ld\n"), (long)i_num, (long)inode.i_blocks);
 			if (flags & FL_VERBOSE)
 				blit();
 			uid = inode.i_uid | (inode.i_uid_high << 16);
@@ -412,11 +411,11 @@ static int ext2_direct_scan(char *device)
 			if (ucheck)
 				add_to_quota(USRQUOTA, i_num, uid, gid,
 					     inode.i_mode, inode.i_links_count,
-					     inode.i_blocks << 9, 0);
+					     ((loff_t)inode.i_blocks) << 9, 0);
 			if (gcheck)
 				add_to_quota(GRPQUOTA, i_num, uid, gid,
 					     inode.i_mode, inode.i_links_count,
-					     inode.i_blocks << 9, 0);
+					     ((loff_t)inode.i_blocks) << 9, 0);
 			if (S_ISDIR(inode.i_mode))
 				dirs_done++;
 			else
@@ -439,7 +438,7 @@ static int ext2_direct_scan(char *device)
  */
 static int scan_dir(char *pathname)
 {
-	struct dirs *dir_stack = { (struct dirs *)NULL };
+	struct dirs *dir_stack = NULL;
 	struct dirs *new_dir;
 	struct dirent *de;
 	struct stat st;
@@ -448,7 +447,7 @@ static int scan_dir(char *pathname)
 	int ret;
 
 	if (lstat(pathname, &st) == -1) {
-		errstr(_("Cannot stat root directory %s: %s\n"), pathname, strerror(errno));
+		errstr(_("Cannot stat directory %s: %s\n"), pathname, strerror(errno));
 		goto out;
 	}
 	qspace = getqsize(pathname, &st);
@@ -475,14 +474,6 @@ static int scan_dir(char *pathname)
 			goto out;
 		}
 
-		qspace = getqsize(de->d_name, &st);
-		if (ucheck)
-			add_to_quota(USRQUOTA, st.st_ino, st.st_uid, st.st_gid, st.st_mode,
-				     st.st_nlink, qspace, !S_ISDIR(st.st_mode));
-		if (gcheck)
-			add_to_quota(GRPQUOTA, st.st_ino, st.st_uid, st.st_gid, st.st_mode,
-				     st.st_nlink, qspace, !S_ISDIR(st.st_mode));
-
 		if (S_ISDIR(st.st_mode)) {
 			if (st.st_dev != cur_dev)
 				continue;
@@ -498,8 +489,15 @@ static int scan_dir(char *pathname)
 			dir_stack = new_dir;
 		}
 		else {
-			debug(FL_DEBUG, _("\tAdding %s size %d ino %d links %d\n"), de->d_name,
-			      st.st_size, st.st_ino, st.st_nlink);
+			qspace = getqsize(de->d_name, &st);
+			if (ucheck)
+				add_to_quota(USRQUOTA, st.st_ino, st.st_uid, st.st_gid, st.st_mode,
+					     st.st_nlink, qspace, 1);
+			if (gcheck)
+				add_to_quota(GRPQUOTA, st.st_ino, st.st_uid, st.st_gid, st.st_mode,
+					     st.st_nlink, qspace, 1);
+			debug(FL_DEBUG, _("\tAdding %s size %Ld ino %d links %d uid %u gid %u\n"), de->d_name,
+			      (long long)st.st_size, (int)st.st_ino, (int)st.st_nlink, (int)st.st_uid, (int)st.st_gid);
 			files_done++;
 		}
 	}
@@ -746,18 +744,52 @@ static int dump_to_file(struct mntent *mnt, int type)
 	return 0;
 }
 
+/* Substract space used by old quota file from usage */
+static void sub_quota_file(struct mntent *mnt, int qtype, int ftype)
+{
+	char *filename;
+	struct stat st;
+	loff_t qspace;
+	struct dquot *d;
+	qid_t id;
+
+	debug(FL_DEBUG, _("Substracting space used by old %s quota file.\n"), type2name(ftype));
+	if (get_qf_name(mnt, ftype, 1 << cfmt, 0, &filename) < 0) {
+		debug(FL_VERBOSE, _("Old %s file not found. Usage will not be substracted.\n"), type2name(ftype));
+		return;
+	}
+
+	if (stat(filename, &st) < 0) {
+		debug(FL_VERBOSE, _("Cannot stat old %s quota file: %s\n"), type2name(ftype), strerror(errno));
+		free(filename);
+		return;
+	}
+	qspace = getqsize(filename, &st);
+	free(filename);
+	
+	if (qtype == USRQUOTA)
+		id = st.st_uid;
+	else
+		id = st.st_gid;
+	if ((d = lookup_dquot(id, qtype)) == NODQUOT) {
+		errstr(_("Quota structure for %s owning quota file not present! Something is really wrong...\n"), type2name(qtype));
+		return;
+	}
+	d->dq_dqb.dqb_curinodes--;
+	d->dq_dqb.dqb_curspace -= qspace;
+	debug(FL_DEBUG, _("Substracted %lu bytes.\n"), (unsigned long)qspace);
+}
+
 /* Buffer quotafile, run filesystem scan, dump quotafiles */
 static void check_dir(struct mntent *mnt)
 {
 	struct stat st;
 	int remounted = 0;
-	loff_t qspace;
 
 	if (lstat(mnt->mnt_dir, &st) < 0)
 		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt, strerror(errno));
 	if (!S_ISDIR(st.st_mode))
 		die(2, _("Mountpoint %s isn't directory?!\n"), mnt);
-	qspace = getqsize(mnt->mnt_dir, &st);
 	cur_dev = st.st_dev;
 	files_done = dirs_done = 0;
 	if (ucheck)
@@ -804,6 +836,14 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 			goto out;
 	}
 	dirs_done++;
+	if (ucheck) {
+		sub_quota_file(mnt, USRQUOTA, USRQUOTA);
+		sub_quota_file(mnt, USRQUOTA, GRPQUOTA);
+	}
+	if (gcheck) {
+		sub_quota_file(mnt, GRPQUOTA, USRQUOTA);
+		sub_quota_file(mnt, GRPQUOTA, GRPQUOTA);
+	}
 	if (flags & FL_VERBOSE)
 		fputs(_("done\n"), stderr);
 	debug(FL_DEBUG | FL_VERBOSE, _("Checked %d directories and %d files\n"), dirs_done,
@@ -831,15 +871,23 @@ static int detect_filename_format(struct mntent *mnt, int type)
 	if (type == USRQUOTA) {
 		if ((option = hasmntopt(mnt, MNTOPT_USRQUOTA)))
 			option += strlen(MNTOPT_USRQUOTA);
+		else if (hasmntopt(mnt, MNTOPT_USRJQUOTA)) {
+			errstr(_("Cannot detect quota format for journalled quota on %s\n"), mnt->mnt_dir);
+			return -1;
+		}
 		else if ((option = hasmntopt(mnt, MNTOPT_QUOTA)))
 			option += strlen(MNTOPT_QUOTA);
 	}
 	else {
 		if ((option = hasmntopt(mnt, MNTOPT_GRPQUOTA)))
 			option += strlen(MNTOPT_GRPQUOTA);
+		else if (hasmntopt(mnt, MNTOPT_GRPJQUOTA)) {
+			errstr(_("Cannot detect quota format for journalled quota on %s\n"), mnt->mnt_dir);
+			return -1;
+		}
 	}
 	if (!option)
-		die(2, _("Can't find quota option on filesystem %s with quotas!\n"), mnt->mnt_dir);
+		die(2, _("Cannot find quota option on filesystem %s with quotas!\n"), mnt->mnt_dir);
 	if (*option == '=')	/* If the file name is specified we can't detect quota format from it... */
 		return -1;
 	snprintf(namebuf, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSV0], extensions[type]);
