@@ -10,7 +10,7 @@
  * 
  * Author:  Marco van Wieringen <mvw@planets.elm.net>
  *
- * Version: $Id: warnquota.c,v 1.12 2003/02/14 18:50:17 jkar8572 Exp $
+ * Version: $Id: warnquota.c,v 1.19 2004/01/09 11:27:45 jkar8572 Exp $
  *
  *          This program is free software; you can redistribute it and/or
  *          modify it under the terms of the GNU General Public License as
@@ -29,6 +29,10 @@
 #include <grp.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <time.h>
+#ifdef USE_LDAP_MAIL_LOOKUP
+#include <ldap.h>
+#endif
 
 #include "mntopt.h"
 #include "pot.h"
@@ -76,6 +80,8 @@
 #define FL_USER 1
 #define FL_GROUP 2
 #define FL_NOAUTOFS 4
+#define FL_SHORTNUMS 8
+#define FL_NODETAILS 16
 
 struct usage {
 	char *devicename;
@@ -83,6 +89,10 @@ struct usage {
 	struct usage *next;
 };
 
+#ifdef USE_LDAP_MAIL_LOOKUP
+LDAP *ldapconn = NULL;
+#endif
+
 struct configparams {
 	char mail_cmd[CNF_BUFFER];
 	char from[CNF_BUFFER];
@@ -94,6 +104,18 @@ struct configparams {
 	char *user_signature;
 	char *group_message;
 	char *group_signature;
+	int use_ldap_mail; /* 0 */
+#ifdef USE_LDAP_MAIL_LOOKUP
+	int ldap_is_setup; /* 0 */
+	char ldap_host[CNF_BUFFER];
+	int ldap_port;
+	char ldap_binddn[CNF_BUFFER];
+	char ldap_bindpw[CNF_BUFFER];
+	char ldap_basedn[CNF_BUFFER];
+	char ldap_search_attr[CNF_BUFFER];
+	char ldap_mail_attr[CNF_BUFFER];
+	char default_domain[CNF_BUFFER];
+#endif /* USE_LDAP_MAIL_LOOKUP */
 };
 
 struct offenderlist {
@@ -115,6 +137,8 @@ struct adminstable {
 };
 
 int qtab_i = 0, fmt = -1, flags;
+char maildev[CNF_BUFFER];
+struct quota_handle *maildev_handle;
 char *configfile = WARNQUOTA_CONF, *quotatabfile = QUOTATAB, *adminsfile = ADMINSFILE;
 char *progname;
 quotatable_t *quotatable;
@@ -126,6 +150,39 @@ struct adminstable *adminstable;
  */
 static struct offenderlist *offenders = (struct offenderlist *)0;
 
+/*
+ * add any cleanup functions here
+ */
+static void wc_exit(int ex_stat)
+{
+#ifdef USE_LDAP_MAIL_LOOKUP
+	if(ldapconn != NULL)
+		ldap_unbind(ldapconn);
+#endif
+	exit(ex_stat);
+}
+
+#ifdef USE_LDAP_MAIL_LOOKUP
+static int setup_ldap(struct configparams *config)
+{
+	int ret;
+
+	ldapconn = ldap_init(config->ldap_host, config->ldap_port);
+	if(ldapconn == NULL) {
+		ldap_perror(ldapconn, "ldap_init");
+		return(-1);
+	}
+
+	ret = ldap_bind_s(ldapconn, config->ldap_binddn, config->ldap_bindpw, LDAP_AUTH_SIMPLE);
+	if(ret < 0) {
+		ldap_perror(ldapconn, "ldap_bind");
+		return(-1);
+	}
+	return(0);
+}
+		
+#endif
+
 struct offenderlist *add_offender(int type, int id, char *name)
 {
 	struct offenderlist *offender;
@@ -172,11 +229,42 @@ void add_offence(struct dquot *dquot, char *name)
 	lptr->usage = usage;
 }
 
+int deliverable(struct dquot *dquot)
+{
+	time_t now;
+	struct dquot *mdquot;
+	
+	if (!maildev[0])
+		return 1;
+
+	time(&now);
+	
+	if (!strcasecmp(maildev, "any") && 
+	   ((dquot->dq_dqb.dqb_bhardlimit && toqb(dquot->dq_dqb.dqb_curspace) >= dquot->dq_dqb.dqb_bhardlimit)
+	   || ((dquot->dq_dqb.dqb_bsoftlimit && toqb(dquot->dq_dqb.dqb_curspace) >= dquot->dq_dqb.dqb_bsoftlimit)
+	   && (dquot->dq_dqb.dqb_btime && dquot->dq_dqb.dqb_btime <= now))))
+		return 0;
+	if (!maildev_handle)
+		return 1;
+	mdquot = maildev_handle->qh_ops->read_dquot(maildev_handle, dquot->dq_id);
+	if (mdquot &&
+	   ((mdquot->dq_dqb.dqb_bhardlimit && toqb(mdquot->dq_dqb.dqb_curspace) >= mdquot->dq_dqb.dqb_bhardlimit)
+	   || ((mdquot->dq_dqb.dqb_bsoftlimit && toqb(mdquot->dq_dqb.dqb_curspace) >= mdquot->dq_dqb.dqb_bsoftlimit)
+	   && (mdquot->dq_dqb.dqb_btime && mdquot->dq_dqb.dqb_btime <= now)))) {
+		free(mdquot);
+		return 0;
+	}
+	free(mdquot);
+	return 1;
+}
+
 int check_offence(struct dquot *dquot, char *name)
 {
 	if ((dquot->dq_dqb.dqb_bsoftlimit && toqb(dquot->dq_dqb.dqb_curspace) >= dquot->dq_dqb.dqb_bsoftlimit)
-	    || (dquot->dq_dqb.dqb_isoftlimit && dquot->dq_dqb.dqb_curinodes >= dquot->dq_dqb.dqb_isoftlimit))
-		add_offence(dquot, name);
+	    || (dquot->dq_dqb.dqb_isoftlimit && dquot->dq_dqb.dqb_curinodes >= dquot->dq_dqb.dqb_isoftlimit)) {
+		if(deliverable(dquot))
+			add_offence(dquot, name);
+	}
 	return 0;
 }
 
@@ -198,11 +286,11 @@ FILE *run_mailer(char *command)
 			close(pipefd[1]);
 			if (dup2(pipefd[0], 0) < 0) {
 				errstr(_("Can't duplicate descriptor: %s\n"), strerror(errno));
-				exit(1);
+				wc_exit(1);
 			}			
 			execl(SHELL, SHELL, "-c", command, NULL);
 			errstr(_("Can't execute '%s': %s\n"), command, strerror(errno));
-			exit(1);
+			wc_exit(1);
 		default:
 			close(pipefd[0]);
 			if (!(f = fdopen(pipefd[1], "w")))
@@ -222,28 +310,114 @@ int mail_user(struct offenderlist *offender, struct configparams *config)
 	FILE *fp;
 	int cnt, status;
 	char timebuf[MAXTIMELEN];
+	char numbuf[3][MAXNUMLEN];
 	struct util_dqblk *dqb;
-	char *to;
+	char *to = NULL;
+#ifdef USE_LDAP_MAIL_LOOKUP
+       	char searchbuf[256];
+	LDAPMessage *result, *entry;
+	BerElement     *ber = NULL;
+	struct berval  **bvals = NULL;
+	int ret;
+	char *a;
+#endif
 
-	if (offender->offender_type == USRQUOTA)
-		to = offender->offender_name;
-	else {
+	if (offender->offender_type == USRQUOTA) {
+#ifdef USE_LDAP_MAIL_LOOKUP
+		if(config->use_ldap_mail != 0) {
+			if((ldapconn == NULL) && (config->ldap_is_setup == 0)) {
+				/* need init */
+				if(setup_ldap(config)) {
+					errstr(_("Could not setup ldap connection, returning.\n"));
+					return -1;
+				}
+				config->ldap_is_setup = 1;
+			}
+
+			if(ldapconn == NULL) {
+				/* ldap was never setup correctly so just use the offender_name */
+				to = sstrdup(offender->offender_name);
+			} else {
+				/* search for the offender_name in ldap */
+				snprintf(searchbuf, 256, "(%s=%s)", config->ldap_search_attr, 
+					offender->offender_name);
+				ret = ldap_search_s(ldapconn, config->ldap_basedn, 
+					LDAP_SCOPE_SUBTREE, searchbuf,
+					NULL, 0, &result);
+				if(ret < 0) {
+					errstr(_("Error with %s.\n"), offender->offender_name);
+					ldap_perror(ldapconn, "ldap_search");
+					return 0;
+				}
+					
+				cnt = ldap_count_entries(ldapconn, result);
+
+				if(cnt > 1) {
+					errstr(_("Multiple entries found for client %s, %d not sending mail.\n"), 
+						offender->offender_name, cnt);
+					return 0;
+				} else if(cnt == 0) {
+					errstr(_("Entry not found for client %s, %d not sending mail.\n"), 
+						offender->offender_name, cnt);
+					return 0;
+				} else {
+					/* get the attr */
+					entry = ldap_first_entry(ldapconn, result);
+					for(a = ldap_first_attribute(ldapconn, entry, &ber); a != NULL;
+						a = ldap_next_attribute( ldapconn, entry, ber)) {
+						if(strcasecmp(a, config->ldap_mail_attr) == 0) {
+							bvals = ldap_get_values_len(ldapconn, entry, a);
+							if(bvals == NULL) {
+								errstr(_("Could not get values for %s.\n"), 
+									offender->offender_name);
+								return 0;
+							}
+							to = sstrdup(bvals[0]->bv_val);
+							break;
+						} 
+					} 
+
+					ber_bvecfree(bvals);
+					if(to == NULL) {
+						/* 
+						 * use just the name and default domain as we didn't find the
+						 * attribute we wanted in this entry
+						 */
+						to = malloc(strlen(offender->offender_name)+
+							strlen(config->default_domain)+1);
+						sprintf(to, "%s@%s", offender->offender_name,
+							config->default_domain);
+					}
+				}
+			}
+		} else {
+			to = sstrdup(offender->offender_name);
+		}
+#else
+		to = sstrdup(offender->offender_name);
+#endif
+	} else {
 		struct adminstable *admin;
 
 		if (!(admin = bsearch(offender->offender_name, adminstable, adminscnt, sizeof(struct adminstable), admin_name_cmp))) {
 			errstr(_("Administrator for a group %s not found. Cancelling mail.\n"), offender->offender_name);
 			return -1;
 		}
-		to = admin->adminname;
+		to = sstrdup(admin->adminname);
 	}
-	if (!(fp = run_mailer(config->mail_cmd)))
+	if (!(fp = run_mailer(config->mail_cmd))) {
+		if(to)
+			free(to);
 		return -1;
+	}
 	fprintf(fp, "From: %s\n", config->from);
 	fprintf(fp, "Reply-To: %s\n", config->support);
 	fprintf(fp, "Subject: %s\n", config->subject);
 	fprintf(fp, "To: %s\n", to);
 	fprintf(fp, "Cc: %s\n", config->cc_to);
 	fprintf(fp, "\n");
+	free(to);
+
 	if (offender->offender_type == USRQUOTA)
 		if (config->user_message)
 			fputs(config->user_message, fp);
@@ -255,37 +429,45 @@ int mail_user(struct offenderlist *offender, struct configparams *config)
 		else
 			fprintf(fp, DEF_GROUP_MESSAGE, offender->offender_name);
 
-	for (lptr = offender->usage; lptr; lptr = lptr->next) {
-		dqb = &lptr->dq_dqb;
-		for (cnt = 0; cnt < qtab_i; cnt++)
-			if (!strcmp(quotatable[cnt].devname, lptr->devicename)) {
-				fprintf(fp, "\n%s (%s)\n", quotatable[cnt].devdesc, quotatable[cnt].devname);
-				break;
-			}
-		if (cnt == qtab_i)	/* Description not found? */
-			fprintf(fp, "\n%s\n", lptr->devicename);
-		fprintf(fp, _("\n                        Block limits               File limits\n"));
-		fprintf(fp, _("Filesystem           used    soft    hard  grace    used  soft  hard  grace\n"));
-		if (strlen(lptr->devicename) > 15)
-			fprintf(fp, "%s\n%15s", lptr->devicename, "");
-		else
-			fprintf(fp, "%-15s", lptr->devicename);
-		if (dqb->dqb_bsoftlimit && dqb->dqb_bsoftlimit <= toqb(dqb->dqb_curspace))
-			difftime2str(dqb->dqb_btime, timebuf);
-		else
-			timebuf[0] = '\0';
-		fprintf(fp, "%c%c%8Lu%8Lu%8Lu%7s",
-		        dqb->dqb_bsoftlimit && toqb(dqb->dqb_curspace) >= dqb->dqb_bsoftlimit ? '+' : '-',
-			dqb->dqb_isoftlimit && dqb->dqb_curinodes >= dqb->dqb_isoftlimit ? '+' : '-',
-			(long long)toqb(dqb->dqb_curspace), (long long)dqb->dqb_bsoftlimit,
-			(long long)dqb->dqb_bhardlimit, timebuf);
-		if (dqb->dqb_isoftlimit && dqb->dqb_isoftlimit <= dqb->dqb_curinodes)
-			difftime2str(dqb->dqb_itime, timebuf);
-		else
-			timebuf[0] = '\0';
-		fprintf(fp, "  %6Lu%6Lu%6Lu%7s\n\n", (long long)dqb->dqb_curinodes,
-		        (long long)dqb->dqb_isoftlimit, (long long)dqb->dqb_ihardlimit, timebuf);
+	if (!(flags & FL_NODETAILS)) {
+		for (lptr = offender->usage; lptr; lptr = lptr->next) {
+			dqb = &lptr->dq_dqb;
+			for (cnt = 0; cnt < qtab_i; cnt++)
+				if (!strcmp(quotatable[cnt].devname, lptr->devicename)) {
+					fprintf(fp, "\n%s (%s)\n", quotatable[cnt].devdesc, quotatable[cnt].devname);
+					break;
+				}
+			if (cnt == qtab_i)	/* Description not found? */
+				fprintf(fp, "\n%s\n", lptr->devicename);
+			fprintf(fp, _("\n                        Block limits               File limits\n"));
+			fprintf(fp, _("Filesystem           used    soft    hard  grace    used  soft  hard  grace\n"));
+			if (strlen(lptr->devicename) > 15)
+				fprintf(fp, "%s\n%15s", lptr->devicename, "");
+			else
+				fprintf(fp, "%-15s", lptr->devicename);
+			if (dqb->dqb_bsoftlimit && dqb->dqb_bsoftlimit <= toqb(dqb->dqb_curspace))
+				difftime2str(dqb->dqb_btime, timebuf);
+			else
+				timebuf[0] = '\0';
+			space2str(toqb(dqb->dqb_curspace), numbuf[0], flags & FL_SHORTNUMS);
+			space2str(dqb->dqb_bsoftlimit, numbuf[1], flags & FL_SHORTNUMS);
+			space2str(dqb->dqb_bhardlimit, numbuf[2], flags & FL_SHORTNUMS);
+			fprintf(fp, "%c%c %7s %7s %7s %6s",
+			        dqb->dqb_bsoftlimit && toqb(dqb->dqb_curspace) >= dqb->dqb_bsoftlimit ? '+' : '-',
+				dqb->dqb_isoftlimit && dqb->dqb_curinodes >= dqb->dqb_isoftlimit ? '+' : '-',
+				numbuf[0], numbuf[1], numbuf[2], timebuf);
+			if (dqb->dqb_isoftlimit && dqb->dqb_isoftlimit <= dqb->dqb_curinodes)
+				difftime2str(dqb->dqb_itime, timebuf);
+			else
+				timebuf[0] = '\0';
+			number2str(dqb->dqb_curinodes, numbuf[0], flags & FL_SHORTNUMS);
+			number2str(dqb->dqb_isoftlimit, numbuf[1], flags & FL_SHORTNUMS);
+			number2str(dqb->dqb_ihardlimit, numbuf[2], flags & FL_SHORTNUMS);
+			fprintf(fp, " %7s %5s %5s %6s\n\n", numbuf[0], numbuf[1], numbuf[2], timebuf);
+		}
 	}
+
+
 	if (offender->offender_type == USRQUOTA)
 		if (config->user_signature)
 			fputs(config->user_signature, fp);
@@ -420,7 +602,13 @@ int readconfigfile(const char *filename, struct configparams *config)
 	sstrncpy(config->cc_to, CC_TO, CNF_BUFFER);
 	sstrncpy(config->support, SUPPORT, CNF_BUFFER);
 	sstrncpy(config->phone, PHONE, CNF_BUFFER);
+	maildev[0] = 0;
 	config->user_signature = config->user_message = config->group_signature = config->group_message = NULL;
+	config->use_ldap_mail = 0;
+
+#ifdef USE_LDAP_MAIL_LOOKUP
+	config->ldap_port = config->ldap_is_setup = 0;
+#endif
 
 	if (!(fp = fopen(filename, "r"))) {
 		errstr(_("Can't open %s: %s\n"), filename, strerror(errno));
@@ -447,7 +635,7 @@ int readconfigfile(const char *filename, struct configparams *config)
 		else {
 			len--;
 			if (buff[len-1] == '\\') {	/* Should join with next line? */
-				bufpos += len-1;
+				bufpos = len-1;
 				continue;
 			}
 		}
@@ -476,7 +664,10 @@ int readconfigfile(const char *filename, struct configparams *config)
 				sstrncpy(config->support, value, CNF_BUFFER);
 			else if (!strcmp(var, "PHONE"))
 				sstrncpy(config->phone, value, CNF_BUFFER);
-			else if (!strcmp(var, "MESSAGE")) {
+			else if (!strcmp(var, "MAILDEV")) {
+				/* set the global */
+				sstrncpy(maildev, value, CNF_BUFFER);
+			} else if (!strcmp(var, "MESSAGE")) {
 				config->user_message = sstrdup(value);
 				create_eoln(config->user_message);
 			}
@@ -492,6 +683,30 @@ int readconfigfile(const char *filename, struct configparams *config)
 				config->group_signature = sstrdup(value);
 				create_eoln(config->group_signature);
 			}
+			else if (!strcmp(var, "LDAP_MAIL")) {
+				if(strcasecmp(value, "true") == 0) 
+					config->use_ldap_mail = 1;
+				else
+					config->use_ldap_mail = 0;
+			}
+#ifdef USE_LDAP_MAIL_LOOKUP
+			else if (!strcmp(var, "LDAP_HOST"))
+				sstrncpy(config->ldap_host, value, CNF_BUFFER);
+			else if (!strcmp(var, "LDAP_PORT"))
+				config->ldap_port = (int)strtol(value, NULL, 10);
+			else if(!strcmp(var, "LDAP_BINDDN"))
+				sstrncpy(config->ldap_binddn, value, CNF_BUFFER);
+			else if(!strcmp(var, "LDAP_BINDPW"))
+				sstrncpy(config->ldap_bindpw, value, CNF_BUFFER);
+			else if(!strcmp(var, "LDAP_BASEDN"))
+				sstrncpy(config->ldap_basedn, value, CNF_BUFFER);
+			else if(!strcmp(var, "LDAP_SEARCH_ATTRIBUTE"))
+				sstrncpy(config->ldap_search_attr, value, CNF_BUFFER);
+			else if(!strcmp(var, "LDAP_MAIL_ATTRIBUTE"))
+				sstrncpy(config->ldap_mail_attr, value, CNF_BUFFER);
+			else if(!strcmp(var, "LDAP_DEFAULT_MAIL_DOMAIN"))
+				sstrncpy(config->default_domain, value, CNF_BUFFER);
+#endif
 			else	/* not matched at all */
 				errstr(_("Error in config file (line %d), ignoring\n"), line);
 		}
@@ -567,6 +782,14 @@ int get_groupadmins(void)
 	return 0;
 }
 
+struct quota_handle *find_handle_dev(char *dev, struct quota_handle **handles)
+{
+	int i;
+
+	for (i = 0; handles[i] && strcmp(dev, handles[i]->qh_quotadev); i++);
+	return handles[i];
+}
+
 void warn_quota(void)
 {
 	struct quota_handle **handles;
@@ -574,39 +797,49 @@ void warn_quota(void)
 	int i;
 
 	if (readconfigfile(configfile, &config) < 0)
-		exit(1);
+		wc_exit(1);
 	if (get_quotatable() < 0)
-		exit(1);
+		wc_exit(1);
 
 	if (flags & FL_USER) {
 		handles = create_handle_list(0, NULL, USRQUOTA, -1, IOI_LOCALONLY | IOI_READONLY | IOI_OPENFILE, (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		if (!maildev[0] || !strcasecmp(maildev, "any"))
+			maildev_handle = NULL;
+		else
+			maildev_handle = find_handle_dev(maildev, handles);
 		for (i = 0; handles[i]; i++)
 			handles[i]->qh_ops->scan_dquots(handles[i], check_offence);
 		dispose_handle_list(handles);
 	}
 	if (flags & FL_GROUP) {
 		if (get_groupadmins() < 0)
-			exit(1);
+			wc_exit(1);
 		handles = create_handle_list(0, NULL, GRPQUOTA, -1, IOI_LOCALONLY | IOI_READONLY | IOI_OPENFILE, (flags & FL_NOAUTOFS ? MS_NO_AUTOFS : 0));
+		if (!maildev[0] || !strcasecmp(maildev, "any"))
+			maildev_handle = NULL;
+		else
+			maildev_handle = find_handle_dev(maildev, handles);
 		for (i = 0; handles[i]; i++)
 			handles[i]->qh_ops->scan_dquots(handles[i], check_offence);
 		dispose_handle_list(handles);
 	}
 	if (mail_to_offenders(&config) < 0)
-		exit(1);
+		wc_exit(1);
 }
 
 /* Print usage information */
 static void usage(void)
 {
-	errstr(_("Usage:\n  warnquota [-ugi] [-F quotaformat] [-c configfile] [-q quotatabfile]\n"));
+	errstr(_("Usage:\n  warnquota [-ugsid] [-F quotaformat] [-c configfile] [-q quotatabfile]\n"));
+	fprintf(stderr, _("Bugs to %s\n"), MY_EMAIL);
+	wc_exit(1);
 }
  
 static void parse_options(int argcnt, char **argstr)
 {
 	int ret;
  
-	while ((ret = getopt(argcnt, argstr, "ugVF:hc:q:a:i")) != -1) {
+	while ((ret = getopt(argcnt, argstr, "ugVF:hc:q:a:isd")) != -1) {
 		switch (ret) {
 		  case '?':
 		  case 'h':
@@ -616,7 +849,7 @@ static void parse_options(int argcnt, char **argstr)
 			break;
 		  case 'F':
 			if ((fmt = name2fmt(optarg)) == QF_ERROR)
-				exit(1);
+				wc_exit(1);
 			break;
 		  case 'c':
 			configfile = optarg;
@@ -636,6 +869,12 @@ static void parse_options(int argcnt, char **argstr)
 		  case 'i':
 			flags |= FL_NOAUTOFS;
 			break;
+		  case 's':
+			flags |= FL_SHORTNUMS;
+			break;
+		  case 'd':
+			flags |= FL_NODETAILS;
+			break;
 		}
 	}
 	if (!(flags & FL_USER) && !(flags & FL_GROUP))
@@ -651,5 +890,6 @@ int main(int argc, char **argv)
 	init_kernel_interface();
 	warn_quota();
 
+	wc_exit(0);
 	return 0;
 }
